# (C) Copyright 2020 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
"""
Version information for this distribution.
This file is autogenerated by the setup.py script.
"""
#: The full version of the package, including a development suffix
#: for unreleased versions of the package.
version = "0.1.0"
#: The Git revision from which this release was made.
git_revision = "8669b51b09e00b4c6b6bd0e9cf8ff83ff96a0d74"
